// SharedMemory.cpp : Master & Slave
// 2016.05 Kyoung Shin Park 
//

#include <windows.h>
#include <iostream>
#include <string.h>
#include <time.h>
#include "SharedMemory.h"
#include "Mutex.h"

#define BUFFER_SIZE 4096

// shared memory
CSharedMemory shmem("_SHMEM_TEST");
HANDLE segment_id = (HANDLE)3782;
int* gCount = NULL;
// mutex
KMutex mut("_MUT_TEST");

bool isMaster = false;
#define MAX_COUNT 30

int randomVal(int min, int max)
{
	static bool first = true;
	if (first)
	{
		srand(time(NULL)); //seeding for the first time only!
		first = false;
	}
	return min + rand() % (max - min);
}

void parseCommandLine(int argc, char ** argv)
{
    for (int i = 1; i < argc; i++) {
		if (!strncmp(argv[i], "--", 2)) {
			if (!strncmp(argv[i], "--master", 8)) {
				isMaster = true;
				printf("MASTER\n");
			}
		}
	}
}

void writeDataTo()
{
	mut.lock();
	(*gCount)++;
	printf("WRITE[%d]%d\n", (int)segment_id, *gCount);
	mut.unlock();
}

void readDataFrom()
{
	mut.lock();
	printf("READ[%d]%d\n", (int)segment_id, *gCount);
	mut.unlock();
}

int main(int argc, char ** argv)
{
	parseCommandLine(argc, argv);

	// init & allocate shared memory segment
	if (isMaster) {
		mut.create();
		segment_id = shmem.getShm((key_t)segment_id, BUFFER_SIZE, S_IRUSR | S_IWUSR | IPC_CREAT);
	}
	else {
		mut.open();
		segment_id = shmem.getShm((key_t)segment_id, BUFFER_SIZE, S_IRUSR | S_IWUSR);
	}
	printf("shmem id=%d\n", (int)segment_id);

	// attach the shared memory segment
	gCount = (int *)shmem.attachShm(segment_id, NULL, 0);
	printf("shared memory attached\n");

	if (isMaster) {
		(*gCount) = 0;
		writeDataTo();
	}

	// while loop
	while (1) {
		printf("    [%d]=%d\n", (int)segment_id, *gCount);
		readDataFrom();

		int s = randomVal(200, 700);
		Sleep(s); // window Sleep

		writeDataTo();
		printf("%d \n\n", *gCount);
	}

	// detach the shared memory segment
	shmem.detachShm(gCount);
	// remove the shared memory segment
	shmem.cleanShm(segment_id, IPC_RMID, NULL);
	// close shared memory mutex
	mut.close();

	printf("shared memory detached & cleanup \n");

	return 0;
}

